% Solves a Linear Programming problem of the form:
%  Maximize c'x
%  subject to a x <= b


% Data definition 

NLAKE = 40;
lakeDepth = 10*sin(pi*[0:NLAKE-1]/(NLAKE-1));



NDATA = 10*NLAKE;

drift = zeros(NDATA,1);
data  = zeros(NDATA,1);
driftMagnitude = 1;
for i=1:NDATA
   r = rand;
   if ((r > 0.95) || ( i == NLAKE ))  
      drift(i:end,1) = driftMagnitude*randn;
   end
end
drift = drift - min(drift);

% Forward Operator
epsDrift = 10;
epsBlocky= 1e-8;
regularization = epsBlocky*[eye(NDATA, NDATA+1) - [zeros(NDATA,1) , eye(NDATA)]](:,1:NDATA);
L = [
     [[eye(NLAKE);eye(NLAKE);eye(NLAKE);eye(NLAKE);eye(NLAKE);eye(NLAKE);eye(NLAKE);eye(NLAKE);eye(NLAKE);eye(NLAKE);], epsDrift*eye(NDATA)];
     [zeros(NDATA,NLAKE), regularization]
     ];
     
% Run ship over the data space     
shiptrack = L(1:NDATA, 1:NLAKE);     
for i=1:NDATA
   data(i) = lakeDepth(  find(shiptrack(i,:)~=0)) + drift(i);
end

     
data = [data;0*data];

NMODEL = size(L,2);
c = [0,ones(1,NMODEL)];

         
a1 = [ ones(size(data)) , - L];
a2 = [ ones(size(data)) , - L];


b1 =  data;
b2 = -data;


a = [a1;a2];
b = [b1;b2];


NPARAMS = size(b,1);

for i=1:NPARAMS
    ctype(i)   = 'U';  % Constraint type
end

for j=1:NMODEL+1
    vartype(j)     = 'C';  % Continuous or Integer
    lower_bound(j) = 0;
    upper_bound(j) = 1e9;
end

GLPK_MIN = 1;
GLPK_MAX = -1;
sense = GLPK_MIN;


[model, optimalValue] = glpk(c,a,b,lower_bound,upper_bound, ctype, vartype, sense);


estimatedDepth = model(1+[1:NLAKE]);
estimatedDrift = model(2+NLAKE:end) * epsDrift;
%estimatedDrift = estimatedDrift - mean(estimatedDrift);

figure(1)
subplot(2,2,1); plot(data(1:NDATA)); title('Data'), xlabel('Sample Point'); ylabel('Depth');
subplot(2,2,2);plot(estimatedDepth, "linewidth", 3); title('Estimated Depth');
subplot(2,2,4);plot([1:NDATA], estimatedDrift, "linewidth", 3, [1:NDATA], drift, "linewidth", 2); title('Estimated Drift');  legend('Estimated', 'True');
subplot(2,2,3);imagesc(shiptrack); title("Ship Track Pattern");
print -deps Fig/lprog-full.eps


figure(2)
plot(data(1:NDATA)); title('Data'), xlabel('Sample Point'); ylabel('Depth');
print -deps Fig/lprog-data.eps


figure(3)
plot(estimatedDepth, "linewidth", 3); title('Estimated Depth');
print -deps Fig/lprog-estimated-depth.eps


figure(4)
plot([1:NDATA], estimatedDrift, "linewidth", 3, [1:NDATA], drift, "linewidth", 2); title('Estimated Drift');  legend('Estimated', 'True');
print -deps Fig/lprog-estimated-drift.eps


figure(5)
imagesc(shiptrack); title("Ship Track Pattern");
print -deps Fig/lprog-shiptrack.eps

